# Working with files and directories

In theory you should be in the `script` directory.  
Check using `pwd` and if it is not the case run `cd ~/{{extra.working_directory}}/script`

![](../images/3-navigation/tree_exo5.png){: style="height:250px"}

## Commands to keep in mind

| Action | Files | Folders |
| :-: | :-: | :-: |
| Inspect | ls | ls |
| View content | cat | ls |
| Navigate to |  | cd |
| Move | mv | mv |
| Copy | cp | cp -r |
| Create | nano | mkdir |
| Delete | rm | rmdir, rm -r |

## Creating directories

We have a `script` directory on one side and a `data` directory in another side. Let's suppose we will run analysis, it will be nice to have an `analysis` dedicated folder.

!!! question "How would you proceed?"

??? example "Click to show the solution"
    `mkdir analysis`

This is quite straightforward.

## Creating files

Let's now create a file... how would you proceed?  
Actually there are 2 main possibilities: 

* using a text editor
* redirecting a command output into a file using the special characters `>` or `>>`. But this is too early to learn this way for now.

There are plenty of text editors available: `ed`, `ex`, `vi`, `vim`, `emacs`, `nano`, etc.

As beginner, a good choice would be `nano`... why? Because the command how to use it are directly discribe in the screen it opens when you call it.

Let's create a file called `protocol.txt` using `nano`:

```bash
nano protocol.txt
```

You will end up in a window that represent the inside of your file, with the cursor ready to listen what you want to write. At the botton of the screen you will see description of few commands available. Indeed the mouse is useless here... everything has to pass by a command, which will be a combinaation of two characters where the first one is always `^` and corresponds to the **control key** on your keybord.

![](../images/4-creation/nano.png){: style="height:250px"}

Try to display the help `^G`. Not easy isnt'it? But it is like that, no way to make it easier... Depending the command line editor the command that can be executed and their key combinations differ. With nano what is conveniant it's that we do not need a command cheatsheets, the commands are described before our very eyes.

Let's now write someting: `I love guacamole`  
and save the result with `^O`

What happens? Actually nano before to quit and save ask you if you are happy with the file name where it will write. Yes sure! So to go ahead just type `↵ Return`. 

!!! question "Why `nano` ask if we wish to keep the file name?"

??? example "Click to show the solution"
    To be sure you don't fo any wrong manipulation and you are on the point to overwrite a file very important. Right, once the modification is done, there is no turning back.

You can now exit with `^X`.

!!! tip
    You can directly type `^X` when you are done with your edition. `nano` will ask first if you want to save the result, then ask if you are sure about the name of the file you are on the way to write in, and then exit.  


## Naming files and directories

There are few rules to respect to make your life smoother.

* No space
* No accent or special symbol! forbidden! `&` `~` `#` `”` `'` `{` `(` `[` `|` `\` `` ` `` `^` `@` `)` `]` `}` `$` `*` `%` `!` `/` `;` `,` `?`
* Use roman letters, numbers and `_` and `-`
* use file extension to guess what is inside `.fa` `.txt` `.bam` changing the extension of a fasta file by the `.bam` extension doesn’t somehow magically turn it into an alignment file. Files merely contain bytes; it’s up to us and our programs to interpret those bytes according. 
* Unix systems are case sensitive. Choose a way to write and follow it! `Sequence.fa` ≠ `SEQUENCE.fa` ≠ `sequence.fa`

## Moving files and directories

The `analysis` directory and the `protocol.txt` file are not located as suitable. We should move them more approriately.

The command `mv` allows to move file and directories. It follow this syntax:  
`mv <source> <target>`  
The difficult task here is to know the path relative/absolute of the source (what we want to move) and the path relative/absolute of the target (where we want to move it).   

Below is the current situation and the situation we want to end up:

![](../images/4-creation/move1.png){: style="height:250px"} ![](../images/4-creation/move2.png){: style="height:250px"}

!!! question "Move the `analysis` folder and `protocol.txt` accordingly."

??? example "Click to show the solution"
    using the absolute path:
    ```bash
    mv /home/<userName>/{{extra.working_directory}}/script/analysis /home/<userName>/{{extra.working_directory}}/analysis
    mv /home/<userName>/{{extra.working_directory}}/script/protocol.txt /home/<userName>/{{extra.working_directory}}/analysis/ 
    ```
    using the relative path:  
    ```bash
    mv analysis ..
    mv protocol.txt ../analysis/
    ```
    You could even mix abolute and relative path...

## Copy files and directories

Let's imagine you want your script `script1.sh` locally to run your analysis, but you want just a copy. You want to keep the original file as it is. 
The command to copy files and directories is `cp`. But for directories you will need to use the `-r` option that stands for recursive (copy also everything inside).

!!! question "Try to copy the `script1.sh` file inside `analysis` folder."

!!! tip 
    Once again you need to well understand where you are in your directory structure and use either the absolute or relative path of the file/folder you want to copy and the place you want to paste it.  
    Remmember that the current directory is also equivqlent to `.`

??? example "Click to show the solution"  
    ```bash
    cp ../script/script1.sh .
    ```

Let's practice a bit more and by trying to copy the entire `script` folder this time.

??? example "Click to show the solution"  
    ```bash
    cp -r ../script .
    ```
    /!\ If you use `cp -r ../script/ .` you will copy everything that is inside but not the `script` folder itself! So you see the difference?      
    Check the result:
    ```bash
    pwd
    ls
    ls script
    ```


### Your first globbing 

Before running you analysis you also need some data. Let's copy in the fastq files present in the `data` folder.
You could use the `cp` command for all the files one by one as we have seen previously.
But imagine you have hundred of those `fastq` files?  
Well Bash does carry out a very powerful feature, the `filename expansion`, a process known as globbing! (to not confound with regular expressions).

Here the list of globbing:

`?` Any single character
`*` Zero or more characters
`[]` Specify a range. Any character of the range or none of them by using ! inside the bracket.
`{term1,term2}` Specify a list of terms separated by commas and each term must be a name or a
wildcard.
`{term1..term2}` Called brace expansion, this syntax
expands all the terms between term1 and term2 (Letters or Integers).

For now you just has to rember the most important one, `*`.

Let's now copy the fastq files using the `*` filename expansion.

??? example "Click to show the solution"  
    ```bash
    cp ../data/*.fastq .
    ```
    Very convenient isn't it?

## Renaming files and directories

Sadly you realize your protocol is bad you will have to write another one. To keep a track of it you just want to rename it.
The easiest way to rename a file or a directory is to use the `mv` command.

!!! question "Change the `protocol.txt` file into `protocol_wrong.txt`."

??? example "Click to show the solution"  
    ```bash
    mv protocol.txt protocol_wrong.txt
    ```

If you were using a path, you just had to follow the same concept. Let's try by renaming the `protocol_wrong.txt` into `protocol_very_wrong.txt` using absolute path:

`mv /home/<userName>/{{extra.working_directory}}/script/protocol_wrong.txt /home/<userName>/{{extra.working_directory}}/script/protocol_very_wrong.txt`

!!! question "Let's finish this exercice by changing the name of the `script` directory into `script_copy`"
??? example "Click to show the solution"  
    ```bash
    mv script script_copy
    ls -Fa # to check everything went well
    ```

## Removing files and directories

Let's clean the mess we have done in this `analysis folder`. The command to remove files and directories is `rm`. In the case of directory you will have to use the `-r` option to make it recursive.

!!! question "remove the `script1.sh` file."

??? example "Click to show the solution"  
    ```bash
    rm script1.sh
    ```

!!! question "Now remove the `fastq` files using the filename expansion."

??? example "Click to show the solution"  
    ```bash
    rm *.fastq
    ```

!!! question "Let's finish this exercice by removing the `script_copy` directory."

??? example "Click to show the solution"  
    ```bash
    rm -r script_copy
    ls -Fa # to check everything went well
    ```

## linking files and directories

Actually in a real world what we have done so far was not very recommended (copy `fastq` files and `scripts.sh` file). 

!!! question "Do you guess why we should have avoid to copy those files?"

??? example "Click to show the solution"  
    Fastq files are generaly huge. Doing a copy double the disk space used by the file, because now you have it twice. It is recommended when possible to avoid to make copies of huge files.
    You may have continue to work on the `script1.sh` file located in the `script/` folder, but what if you forgot to copy the last version of it into your `analysis` directory?

There is a solution ease your life, giving you the simplicity to have a file locally (no need to think about the path) and to avoid to have a copy of the file. The **link**.

It exists the `hard link` and the `soft link`. 

xxxxxx

## Autocompletion

You did realize how painfull it is to write paths? Actually bash is full of tricks and maybe you already found the `autocompletion` one. What it is and how it works?
It is kind of easy and you will use it evertime...

xxxx